/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.Player;
import java.util.UUID;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public final class Game {
    @XmlAttribute
    private String white;
    @XmlAttribute
    private String black;
    @XmlID
    private String id;

    private transient Board board;

    Game() {
    }

    public Game(String first, String second) {
        this.white = first;
        this.black = second;
        this.id = UUID.randomUUID().toString();
    }

    public String getId() {
        return id;
    }

    public String getWhite() {
        return white;
    }

    public String getBlack() {
        return black;
    }

    public Board getBoard() {
        if (board == null) {
            board = Board.empty();
        }
        return board;
    }

    public void apply(String player, Move m) throws IllegalPositionException {
        Player p = null;
        if (getWhite().equals(player)) {
            p = getBoard().getPlayers().get(0);
        } else {
            if (getBlack().equals(player)) {
                p = getBoard().getPlayers().get(1);
            }
        }
        if (p != getBoard().getCurrentPlayer()) {
            throw new IllegalArgumentException("Wrong player: " + player);
        }

        board = getBoard().apply(m);
    }
}
