/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor.resources;

import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.webidor.*;
import com.sun.jersey.spi.resource.Singleton;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.Move;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/")
@Singleton
public final class Quoridor {
    private List<Game> games = new ArrayList<Game>();

    @POST
    @Path("games")
    public Game createGame(@QueryParam("white") String user1, @QueryParam("black") String user2) {
        Game g = new Game(user1, user2);
        games.add(g);
        return g;
    }

    @PUT
    @Path("games/{id}")
    public Game applyMove(@PathParam("id") String id, @QueryParam("player") String player, @QueryParam("move") String move)
    throws IllegalPositionException {
        Game g = findGame(id);
        if (g == null) {
            throw new IllegalArgumentException("Unknown game " + id);
        }
        Move m = Move.valueOf(move);
        g.apply(player, m);
        return g;
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("games")
    public List<Game> getGames() {
        return games;
    }

    private Game findGame(String id) {
        for (Game g : games) {
            if (g.getId().equals(id)) {
                return g;
            }
        }
        return null;
    }


    //
    // start the server
    //

    public static void main(String[] args) throws IOException {

        final String baseUri = "http://localhost:9998/";
        final Map<String, String> initParams = new HashMap<String, String>();

        initParams.put("com.sun.jersey.config.property.packages",
                "cz.xelfi.quoridor.webidor.resources");

        System.out.println("Starting HttpServer...");
        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.webidor");
        HttpServer threadSelector = HttpServerFactory.create(baseUri, rc);
        threadSelector.start();
        System.out.println(String.format(
                    "Jersey app started with WADL available at %sapplication.wadl\n" +
            "Try out %shelloworld\nHit enter to stop it...", baseUri, baseUri));
            System.in.read();
            threadSelector.stop(0);
            System.exit(0);
    }

}
